#include <unistd.h>
#include <signal.h>
#include "forms.h"
#include "main_panel.h"
#include "setup_panel.h"
#include "playlist.h"
#include "about_panel.h"
#include "xpm.h"
#include "engine.h"

/* signal callback */

static void sig_chld(int sig)
{
	if ( !mixer_pid ) printf("Warning : got unexpected SIGCHLD\n");
	else 
	{
		mixer_pid = 0;
		fl_set_button(panel->mixer_button, 0);
	}
}
		
/* callbacks for form main_panel */

int about_handler(FL_OBJECT * obj, int event, FL_Coord mx, FL_Coord my,
		   int key, void * data)
{
	if ( event == FL_RELEASE ) 
	{
		fl_hide_form(about->about_panel);
		about->active = 0;
	} 
	if ( event == FL_PUSH )		
	{
		fl_show_form(about->about_panel, FL_PLACE_FREE, FL_FULLBORDER, "Mod 4 X Info Panel");
		about->active = 1;
	}
}

void list_button_callback(FL_OBJECT * ob, long data)
{
	if ( list->active )
	{
		fl_hide_form(list->playlist);
		list->active = 0;
		if ( list->updated )
		{
			refresh();
			list->updated = 0;
		}
	} else
	{
		fl_show_form(list->playlist, FL_PLACE_FREE, FL_FULLBORDER, "Mod 4 X Playlist");
		list->active = 1;
	}
}

void setup_button_callback(FL_OBJECT * ob, long data)
{
	if ( setup->active )
	{
		fl_hide_form(setup->setup_panel);
		setup->active = 0;
	} else
	{
		fl_show_form(setup->setup_panel, FL_PLACE_FREE, FL_FULLBORDER,"Mod 4 X Setup");
		setup->active = 1;
	}

}

void mixer_button_callback(FL_OBJECT * ob, long data)
{	
	if ( !mixer_pid )
	{
		mixer_pid = fork();
		if ( mixer_pid == -1 ) printf("Cannot fork program\n");
		if ( mixer_pid == 0 ) 
			if ( execl(config->mixer_path, NULL) == -1 ) 
			{
				printf("Cannot exec mixer\n");
				abort();
			}
		signal(SIGCHLD, sig_chld);
	} else 
		if ( kill(mixer_pid, 15) == -1 ) printf("Cannot terminate mixer process\n");
}

void quit_button_callback(FL_OBJECT * ob, long data)
{
	
}

int led_panel_handler(FL_OBJECT * obj, int event, FL_Coord mx,
		      FL_Coord my, int key, void * xev)
{
	int	i,j;
	
	if ( event == FL_DRAW ) update_display();
	if ( (event == FL_STEP) & is_playing() ) update_pos(mp_sngpos);
	return 0;
}

int control_handler(FL_OBJECT * obj, int event, FL_Coord mx, FL_Coord my,
		    int key, void * xev)
{
	if ( !( (event == FL_PUSH)  || (event == FL_RELEASE) ) )
		return 0;
	
	if ( obj == panel->stop_button )
	{
		if ( event == FL_PUSH ) stop();
		else panel->status &= ~SF_STOP;
	} else
	if ( obj == panel->play_button )
	{
		if ( event == FL_PUSH ) play();
		else panel->status &= ~SF_PLAY;
	} else
	if ( obj == panel->skip_prev_button )
	{
		switch ( event )
		{
			case FL_PUSH :
				fl_set_pixmapbutton_data(panel->skip_prev_button, skip_prev_on_xpm);
				panel->status |= SF_SKIP_PREV;
				skip_prev();
				break;
			case FL_RELEASE :
				fl_set_pixmapbutton_data(panel->skip_prev_button, skip_prev_off_xpm);
				panel->status &= ~SF_SKIP_PREV;
				break;
		}
	} else
	if ( obj == panel->rew_button )
	{
		switch ( event )
		{
			case FL_PUSH :
				fl_set_pixmapbutton_data(panel->rew_button, rew_on_xpm);
				panel->status |= SF_REW;
				rew();
				break;
			case FL_RELEASE :
				fl_set_pixmapbutton_data(panel->rew_button, rew_off_xpm);
				panel->status &= ~SF_REW;
				break;
		}
	} else
	if ( obj == panel->ff_button )
	{
		switch ( event )
		{
			case FL_PUSH :
				fl_set_pixmapbutton_data(panel->ff_button, ff_on_xpm);
				panel->status |= SF_FF;
				ff();
				break;
			case FL_RELEASE :
				fl_set_pixmapbutton_data(panel->ff_button, ff_off_xpm);
				panel->status &= ~SF_FF;
				break;
		}
	} else
	if ( obj == panel->skip_next_button )
	{
		switch ( event )
		{
			case FL_PUSH :
				fl_set_pixmapbutton_data(panel->skip_next_button, skip_next_on_xpm);
				panel->status |= SF_SKIP_NEXT;
				skip_next();
				break;
			case FL_RELEASE :
				fl_set_pixmapbutton_data(panel->skip_next_button, skip_next_off_xpm);
				panel->status &= ~SF_SKIP_NEXT;
				break;
		}
	}	
	return 0;
}
